'use strict';

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Immutable = require('immutable');
var ulid = require('ulid');

var _require = require('./queryset'),
    QuerySet = _require.QuerySet;

var _require2 = require('./manager'),
    createObjectManagerConstructor = _require2.createObjectManagerConstructor;

var _require3 = require('./emitter'),
    createChangesEmitter = _require3.createChangesEmitter;

/*
 * Default setup
 */

var PRIMARY_KEY_NAME = '_pk';

var Entry = function () {
  function Entry(_ref) {
    var values = _ref.values,
        setup = _ref.setup,
        model = _ref.model;
    (0, _classCallCheck3.default)(this, Entry);

    this.values = values;
    this.setup = setup;
    this.model = model;
  }

  (0, _createClass3.default)(Entry, [{
    key: 'assign',
    value: function assign() {
      var rawValues = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      Object.assign(this, rawValues);
    }
  }, {
    key: 'save',
    value: function save() {
      this.values = this.manager.sync(this.values);
      return this;
    }
  }, {
    key: 'update',
    value: function update() {
      var rawValues = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      this.assign(rawValues);
      return this.save();
    }
  }, {
    key: 'delete',
    value: function _delete() {
      this.values = this.manager.unsync(this.values);
      return this;
    }
  }, {
    key: 'toJSON',
    value: function toJSON() {
      return {};
    }
  }, {
    key: 'equals',
    value: function equals(_ref2) {
      var values = _ref2.values;

      return this.values === values;
    }
  }, {
    key: 'manager',
    get: function get() {
      return this.model.objects;
    }
  }, {
    key: 'pk',
    get: function get() {
      var pkName = this.setup.get('primaryKeyName');
      return this.values.get(pkName);
    }
  }]);
  return Entry;
}();

var createObjectsConstructor = function createObjectsConstructor(_ref3) {
  var setup = _ref3.setup;

  var QuerySetConstructor = setup.get('QuerySetConstructor');
  return createObjectManagerConstructor(QuerySetConstructor);
};

var defaultSetup = {
  EntryConstructor: Entry,
  QuerySetConstructor: QuerySet,
  objectsConstructorFactory: createObjectsConstructor,
  primaryKeyName: PRIMARY_KEY_NAME
};

/*
 * Composition
 */

var applyMixin = function applyMixin(_ref4) {
  var name = _ref4.name,
      setup = _ref4.setup,
      mixin = _ref4.mixin;
  return mixin({ Base: setup.get(name), setup: setup }, { setup: setup });
};

var createMixin = function createMixin(_ref5) {
  var name = _ref5.name,
      mixin = _ref5.mixin;
  return function (_ref6) {
    var setup = _ref6.setup;
    return setup.set(name, applyMixin({ name: name, setup: setup, mixin: mixin }));
  };
};

var composeSetup = function composeSetup(_ref7) {
  var model = _ref7.model,
      _ref7$setup = _ref7.setup,
      setup = _ref7$setup === undefined ? Immutable.Map(defaultSetup) : _ref7$setup,
      _ref7$composers = _ref7.composers,
      composers = _ref7$composers === undefined ? [] : _ref7$composers;

  var composer = Immutable.List(composers).get(0);

  return composer ? composeSetup({
    model: model,
    setup: composer({ model: model, setup: setup }),
    composers: Immutable.List(composers).shift(0)
  }) : setup;
};

/*
 * API
 */

var createEntryMixin = function createEntryMixin(mixin) {
  return createMixin({ name: 'EntryConstructor', mixin: mixin });
};
var createQuerySetMixin = function createQuerySetMixin(mixin) {
  return createMixin({ name: 'QuerySetConstructor', mixin: mixin });
};
var createObjectManagerMixin = function createObjectManagerMixin(mixin) {
  return function (_ref8) {
    var setup = _ref8.setup;

    var parentFactory = setup.get('objectsConstructorFactory');

    return setup.set('objectsConstructorFactory', function (_ref9) {
      var _ref9$newSetup = _ref9.newSetup,
          newSetup = _ref9$newSetup === undefined ? setup : _ref9$newSetup,
          model = _ref9.model;
      return mixin({ Base: parentFactory({ setup: newSetup }), setup: newSetup, model: model });
    });
  };
};

var createModel = function createModel(composers) {
  var emitter = createChangesEmitter();
  var model = { emitter: emitter, id: ulid() };

  var objects = null;
  var setup = composeSetup({
    model: model,
    setup: Immutable.Map(defaultSetup),
    composers: Array.isArray(composers) ? composers : [composers]
  });

  model.inject = function (injected) {
    setup = composeSetup({
      model: model,
      setup: setup,
      composers: Array.isArray(injected) ? injected : [injected]
    });
  };

  Object.defineProperty(model, 'objects', {
    get: function get() {
      var opts = objects || { model: model, setup: setup, emitter: emitter, values: Immutable.Map() };

      var objectsConstructorFactory = setup.get('objectsConstructorFactory');
      var ObjectManagerConstructor = objectsConstructorFactory({ setup: setup, model: model });

      objects = new ObjectManagerConstructor(opts);

      return objects;
    }
  });

  return model;
};

var mix = function mix() {
  var composers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  return function (_ref10) {
    var setup = _ref10.setup;
    return composeSetup({ setup: setup, composers: composers });
  };
};

module.exports = {
  createEntryMixin: createEntryMixin,
  createQuerySetMixin: createQuerySetMixin,
  createObjectManagerMixin: createObjectManagerMixin,
  createModel: createModel,
  mix: mix
};
